//*****************************************
// AT mega328 Two Nixie Clock Program
// CPU Clock:8MHz
//   iq[Yf[^@WȂBIV[^(8MHz)ɂj
//
// File Name:nixie.c
//
//  2016/03/26 Taishi
//*****************************************

#include	<avr/pgmspace.h>
#include	<avr/io.h>

#include	"two_nixie.h"
#include	"nixie.h"

// **********************************************************
// *********************** NIXE TUBE ************************
// **********************************************************
//

// **********************************************************
//@jLV[ǂ͂O`Xƍ_\
//@@jLV[ǑI
//@@@ŏʁ@obQfgfiݖgpj
//@@@Qʁ@obRfgfiݖgpj
//@@@Rʁ@obSfgf
//@@@ŉʁ@obTfgf
//
//@@\locSCTCUCV̂Srbĝabcōs
//@@@V@U@T@S
//@@@k@k@k@k@||@O
//@@@k@k@k@g@||@P
//@@@k@k@g@k@||@Q
//@@@k@k@g@g@||@R
//@@@k@g@k@k@||@S
//@@@k@g@k@g@||@T
//@@@k@g@g@k@||@U
//@@@k@g@g@g@||@V
//@@@g@k@k@k@||@W
//@@@g@k@k@g@||@X
//@@@g@k@g@k@||@
//@@@g@k@g@g@||@
//@@@g@g@k@k@||@
//@@@g@g@k@g@||@
//@@@g@g@g@k@||@
//@@@g@g@g@g@||@
//
//@@_\
//@@@ocPFgœ_Akŏ
//
// **********************************************************

static	unsigned char	nixie_data[4];
static	unsigned char	nixie_dot[4];
static	unsigned char	nixie_select[4];
static	unsigned char	nixie_act_num = 0;
static	unsigned char	dynamic_nixie = 0;
static	unsigned char	step_dynamic_nixie;				// jLV[ǃ_Ci~bN\p

#define		NIXE_DATA_CLR		0x0d	// PD 1,4--7 Clear
#define		NIXE_SELECT_CLR		0xc3	// PC2 -- 5 Clear
#define		NIXE_NUMBER_OFF		0xf0	// PD4--7:'H'
#define		NIXE_DOT_ON			0xf2	// PD1:'H' PD4--PD7 ̂g͐\
#define		NIXE_DOT_OFF		0xf0	// PD1:'L' PD4--PD7 ̂g͐\
//******************************
//  Nixe\@
// *****************************
void	init_nixe(void)
{
	unsigned char	uc;

	PORTD &= NIXE_DATA_CLR;						// ȂA_Ȃ
	PORTC &= NIXE_SELECT_CLR;					// IȂ
	DDRD |= ((1 << DDD1) | (1 << DDD4) | (1 << DDD5) | (1 << DDD6) | (1 << DDD7));		// Port-D Bit1,4567 : Output
	DDRC |= ((1 << DDC2) | (1 << DDC3) | (1 << DDC4) | (1 << DDC5));					// Port-C Bit2345: Output

// \A_ f[^
	for(uc = 0;uc < 4; uc++){
		nixie_dot[uc] = NIXE_DOT_OFF;			// _NA
		nixie_data[uc] = NIXE_NUMBER_OFF;		// NA
	}

// I f[^
	nixie_select[0] = 0x04;				// ŏ
	nixie_select[1] = 0x08;				// Q
	nixie_select[2] = 0x10;				// R
	nixie_select[3] = 0x20;				// ŉ


	nixie_act_num = 0;

	dynamic_nixie = 0;

	step_dynamic_nixie = 0;				// jLV[ǃ_Ci~bN\p

}
//******************************
// jLV[Ǖ\
// sQ̊荞݂ŌĂ΂
//******************************
void	proc_dynamic_nixie(void)
{
	if(dynamic_nixie == 1){
		PORTC &= NIXE_SELECT_CLR;					// IȂ
		PORTD &= NIXE_DATA_CLR;						// ȂA_Ȃ

		if((nixie_act_num & 1)==0){					// _̎
			PORTD |= nixie_dot[nixie_act_num/2];		// _
		}else{
			PORTD |= nixie_data[nixie_act_num/2];		// 
		}
		PORTC |= nixie_select[nixie_act_num/2];		// I

		nixie_act_num++;
		nixie_act_num &= 0x07;
	}
}

//******************************
// Dynamic Nixie display Active/Non active
//  on : 1= Active, other=Non Active
// *****************************
void	dynamic_nixie_active(unsigned char on)
{
	dynamic_nixie = on;
}
//******************************
// display Nixie Si_tj
//  cp : W̕
// *****************************
void	disp_nixie_full(char *cp)
{
	unsigned char	i;

	for(i=0;i<8;i++){
		if(*(cp+i)==0x00){
			break;
		}else{
			if( (i%2)==0 ){					// _
				if(*(cp+i)=='.'){
					nixie_data[i] = NIXE_DOT_ON;
				}else if(*(cp+i)==','){
					nixie_data[i] = NIXE_DOT_ON;
				}else if(*(cp+i)==':'){
					nixie_data[i] = NIXE_DOT_ON;
				}else{
					nixie_data[i] = NIXE_DOT_OFF;
				}
			}else{							// 
				if( ('0' <=  *(cp+i) )&&(*(cp+i) <= '9') ){
					nixie_data[i] = (( *(cp+i) ) & 0x0f) << 4;
				}else{
					nixie_data[i] = NIXE_NUMBER_OFF;
				}
			}
		}
	}
}
//******************************
// display Nixie Si_tj
//  cp : őW̕(' '(󔒁j'0' -- '9''.'(hbgjɌ
// *****************************
void	disp_nixie_number(char *cp)
{
	unsigned char	i;
	unsigned char	k;
	unsigned char	d;

	k=0;
	d=0;
	for(i=0;i<8;i++){
		if(*(cp+i)==0x00){
			break;
		}else{
			if(*(cp+i)=='.'){
				d = 1;
			}else{
				if( ('0' <=  *(cp+i) )&&(*(cp+i) <= '9') ){
					nixie_data[k*2+1] = (( *(cp+i) ) & 0x0f) << 4;
				}else{
					nixie_data[k*2+1] = NIXE_NUMBER_OFF;
				}
				if(d == 0){
					nixie_data[k*2+0] = NIXE_DOT_OFF;
				}else{
					nixie_data[k*2+0] = NIXE_DOT_ON;
				}
				d=0;
				k++;
				if(3 < k){
					break;
				}
			}
		}
	}
}
//******************************
// Nixie _\
//       k : ʒuiO`RAOFŏʁj
//  on_off : 1=ON, other=OFFNIXE_DOT_ON or NIXE_DOT_OFF
// *****************************
void	disp_dynamic_nixie_dot(unsigned char k, unsigned char on)
{
	if(on == 1){
		nixie_dot[k] = NIXE_DOT_ON;
	}else{
		nixie_dot[k] = NIXE_DOT_OFF;
	}
}
//******************************
// Nixie \
//  k : ʒuiO`RAOFŏʁj
//  n : O`XAPTiPT͋󔒁j
// *****************************
void	disp_dynamic_nixie_number(unsigned char k, unsigned char n)
{
	nixie_data[k] = ((n << 4) & 0xf0);
}

//******************************
// Nixe _\iX^eBbN\j
//       k : ʒuiO`RAOFŏʁj
//  on_off : NIXE_DOT_ON or NIXE_DOT_OFF
//  ܂ő̌ŕ\Ắ@̕\i_j
// *****************************
void	disp_static_nixie_dot(unsigned char k)
{
	PORTC &= NIXE_SELECT_CLR;					// IȂ
	PORTD &= NIXE_DATA_CLR;						// ȂA_Ȃ

	PORTD |= NIXE_DOT_ON;						// _nm
	PORTC |= nixie_select[k];					// I

}
//******************************
// Nixie \iX^eBbN\j
//  k : ʒuiO`RAOFŏʁj
//  n : O`XAPTiPT͋󔒁j
//  ܂ő̌ŕ\Ắ@̕\i_j
// *****************************
void	disp_static_nixie_number(unsigned char k, unsigned char n)
{
	PORTC &= NIXE_SELECT_CLR;					// IȂ
	PORTD &= NIXE_DATA_CLR;						// ȂA_Ȃ

	PORTD |= (n << 4);							// 
	PORTC |= nixie_select[k];					// I
}
